#ifndef __SPI_H
#define __SPI_H
#include "stm32f0xx.h"
#include "gpio.h"
#include <stdint.h>
#define SPI_BUFFER_LENGTH 32


namespace spi {

        //Definition here so we can store a pointer to the class in the SPI class
        class Device;

        class SPI {
        public:
                SPI(void);
                ~SPI(void);
                // Clock, GPIO and SPI Interrupt needs to be initiated seperately
                // You also need to make an SPI Interrupt Handler function
                // Init SPI to run with phase, polarity = 0,0
                void Init(SPI_TypeDef *Instance);
                uint8_t TradeByte(uint8_t transmit);
                //Should be called by the real interrupt handler (in usercode)
                void InterruptHandler(void);
                //Returns error if theres already a transfer going on
                uint8_t StartTransfer(uint8_t *out, uint8_t *in, uint8_t length, Device *callingDevice);
                //Returns 0 if no transfer is ongoing
                uint8_t GetTransferStatus(void);
        private:
                SPI_TypeDef *Instance;
                uint8_t bufferLength;
                uint8_t bufferInPos;
                uint8_t bufferOutPos;
                uint8_t *bufferIn;
                uint8_t *bufferOut;
                Device *d;
        };

        class Device {
        public:
                //Configures Pin (clock needs to be enabled externally)
                Device(void);
                ~Device(void);
                void Init(SPI &SPIInstance, gpio::pins pin = gpio::PA4);
                //returns 2 if there is a still non completed transfer in device, 1 if there is a transfer within SPI,
                // 3 if data is too long
                uint8_t StartTransfer(void (*transferCallback)(void),uint8_t *sendData, uint8_t length);
                //This is a simple blocking call to exchange 1 byte
                //(As it only calls the SPI class function with the same name, you need to manually select your slave)
                uint8_t TradeByte(uint8_t transmit);
                void Select(void);
                void Deselect(void);
                //Should only be called by SPI class
                void TransferEndCallback(void);
                //Returns 0 if there is no data, otherwise the length
                uint8_t GetData(uint8_t *data);
                //Returns 0 if no transfer is ongoing, 1 if there is one ongoing, 2 if there is still unchecked data
                uint8_t GetTransferStatus(void);
        private:
                SPI *SPIInstance;
                gpio::pins pin;
                //Status 0b10->Old data, 0b1 -> transfer ongoing, 0 -> Nothing going on
                uint8_t status;
                uint8_t bufferOut[SPI_BUFFER_LENGTH];
                uint8_t bufferIn[SPI_BUFFER_LENGTH];
                uint8_t transferLength;
                void (*transferCallback)(void);
        };
        //Enables SPI and GPIO clock, configures PA5,PA6,PA7
        void specificInitDefault(void);
}

#endif
